function Player() {
    this.targetAngle = this.angle = this.img = this.isFlying = this.activePlanet = undefined;
    this.pastPoint = this.movePlus = this.activePlanetSpeed = false;
    this.xDiference = this.yDiference = this.pointsToPlus = undefined;
    this.setView()
}

Player.prototype.setView = function() {
    this.img = new paper.Raster(variables.playerImage);
    this.img.size.height = settings.PLAYER_HEIGHT;
    this.img.size.width = variables.playerImage.width * (settings.PLAYER_HEIGHT / variables.playerImage.height);
}

Player.prototype.setToStart = function() {
    this.xDiference = this.yDiference = undefined;
    this.targetAngle = undefined;
    this.isFlying = false;
    this.pointsToPlus = settings.START_POINTS_PER_FLIGHT_FRAME;
    this.angle = 270;
    this.setAngleToPlanet();
}

Player.prototype.setAngleToPlanet = function() {
    this.img.matrix.rotation = this.angle + 90;
    var distance = this.activePlanet.size / 2 + settings.PLAYER_HEIGHT / 2;
    this.img.position = new paper.Point(this.activePlanet.img.position.x + distance * Math.cos((this.angle)* Math.PI / 180 )
            , this.activePlanet.img.position.y + distance * Math.sin((this.angle) * Math.PI / 180));
}

Player.prototype.setFlightDegree = function(degree) {
    if (degree < 0) degree = Math.round(360 - (180 - degree) + 180);
    this.movePlus = false;
    var distDegree = degree - this.angle;
    if(distDegree < 0 ) distDegree += 360;
    if(distDegree < 180) this.movePlus = true;
    this.targetAngle = degree;
    this.isFlying = true;
}

Player.prototype.landToPlanet = function(planet) {
    this.isFlying = false;
    this.activePlanet = planet;
    this.activePlanetSpeed = settings.FLIGHT_SPEED;

    var deltaY = this.img.position.y - planet.img.position.y;
    var deltaX = this.img.position.x - planet.img.position.x;
    var degree = Math.atan2(deltaY, deltaX) * 180 / Math.PI;
    if (degree < 0) degree = Math.round(360 - (180 - degree) + 180);
    this.angle = degree;

    this.setAngleToPlanet();
}

Player.prototype.setFlyAway = function() {
        this.targetAngle = undefined;
        this.activePlanet = undefined;
}

Player.prototype.onFrame = function() {
    if(typeof (this.activePlanet) != "undefined") {
        this.setActivePlanetToCenter();
    }

    this.xDiference = this.yDiference = undefined;
    if(typeof (this.targetAngle) != "undefined") {
        this.angle = this.movePlus ? this.angle + settings.ANGLE_SPEED : this.angle - settings.ANGLE_SPEED;
        this.setAngleToPlanet();
        if(this.angle > 360) this.angle -= 360;
        if(this.angle < 0) this.angle += 360;
//        if(Math.abs(this.angle - this.targetAngle) <= settings.ANGLE_SPEED) {
//            this.angle = this.targetAngle;
//            this.setAngleToPlanet();
//            this.targetAngle = undefined;
//            this.activePlanet = undefined;
//        }
        return;
    }

    if(!this.isFlying) return;
    this.pastPoint = this.img.position;
    this.img.position = new paper.Point(this.img.position.x + settings.FLIGHT_SPEED * Math.cos((this.angle)* Math.PI / 180 )
            , this.img.position.y + settings.FLIGHT_SPEED * Math.sin((this.angle) * Math.PI / 180));
    this.xDiference = this.pastPoint.x - this.img.position.x;
    this.yDiference = this.pastPoint.y - this.img.position.y;
    this.img.position = this.pastPoint;
    objects.game.points += this.pointsToPlus;
    objects.game.setTexts();
}

Player.prototype.setActivePlanetToCenter = function() {
    if(typeof (this.activePlanet.img) == "undefined") return;
    if(this.activePlanet.img.position.x == constants.GAME_WIDTH / 2 && this.activePlanet.img.position.y == constants.GAME_HEIGHT / 2) return;
    var midPoint = new paper.Point(constants.GAME_WIDTH / 2, constants.GAME_HEIGHT / 2);
    var distance = this.activePlanet.img.position.getDistance(midPoint);
    if(distance < this.activePlanetSpeed / 2) {
        this.activePlanet.img.position = midPoint;
        this.setObjectsToPoint(midPoint);
        return;
    }
    if(this.activePlanet.img.position.getDistance(midPoint) < this.activePlanetSpeed * 3) this.activePlanetSpeed /= 1.25;

    var L = this.activePlanetSpeed / distance;
    var newActiveX = (1 - L) * this.activePlanet.img.position.x + L * midPoint.x;
    var newActiveY = (1 - L) * this.activePlanet.img.position.y + L * midPoint.y;
    this.setObjectsToPoint(new paper.Point(newActiveX, newActiveY));
}

Player.prototype.setObjectsToPoint = function(newPoint) {
    var differenceX = newPoint.x - this.activePlanet.img.position.x;
    var differenceY = newPoint.y - this.activePlanet.img.position.y;

    this.activePlanet.img.position = newPoint;
    this.activePlanet.mapImg.position = new paper.Point(this.activePlanet.img.position.x / 20 + constants.GAME_WIDTH - settings.ACTIVE_PLANETS_FIELD_SIZE / 10 / 2 - constants.GAME_WIDTH / 10 / 4,
            this.activePlanet.img.position.y /20 + constants.GAME_HEIGHT - settings.ACTIVE_PLANETS_FIELD_SIZE / 10 * 2 + settings.ACTIVE_PLANETS_FIELD_SIZE / 10 / 2 - constants.GAME_HEIGHT / 10 / 4);
    this.img.position.x += differenceX;
    this.img.position.y +=differenceY;

    var i = 0;
    while(i < objects.game.space.planets.length) {
        if(objects.game.space.planets[i] == this.activePlanet) {
            i++;
            continue;
        }
        objects.game.space.planets[i].img.position.x += differenceX;
        objects.game.space.planets[i].img.position.y += differenceY;
        objects.game.space.planets[i].mapImg.position = new paper.Point(objects.game.space.planets[i].img.position.x / 20 + constants.GAME_WIDTH - settings.ACTIVE_PLANETS_FIELD_SIZE / 10 / 2 - constants.GAME_WIDTH / 10 / 4,
                objects.game.space.planets[i].img.position.y /20 + constants.GAME_HEIGHT - settings.ACTIVE_PLANETS_FIELD_SIZE / 10 * 2 + settings.ACTIVE_PLANETS_FIELD_SIZE / 10 / 2 - constants.GAME_HEIGHT / 10 / 4);

        if (objects.game.space.planetOutsideField(objects.game.space.planets[i])) {
            var planetPoint = objects.game.space.planets[i].img.position;
            objects.game.space.planets[i].clear();
            objects.game.space.planets.splice(i, 1);
            objects.game.space.setNeededPlanets(planetPoint);
            continue;
        }
        i++;
    }
}