function Space() {
    this.neededPlanets = this.planets = undefined;
    this.map = new Map();
    this.nova = new Nova();
}

Space.prototype.setToStart = function() {
    this.nova.setToStart();
    this.neededPlanets = settings.START_PLANETS_COUNT;
    this.planets = new Array();
    var planet = new Planet();
    this.planets.push(planet);
    objects.game.player.activePlanet = planet;
    planet.setToPoint(new paper.Point(constants.GAME_WIDTH / 2, constants.GAME_HEIGHT / 2));
    planet.img.moveBelow(this.map.backRect);
    planet.img.moveBelow(this.map.viewRect);
    this.setNeededPlanets();
}

Space.prototype.setNeededPlanets = function(pastPoint) {
    var attempts = 0;
    while (this.planets.length < this.neededPlanets) {
        attempts++;
        if(attempts > settings.MAX_PLANET_ADD_ATTEMPTS) break;
        if(typeof (pastPoint) == "undefined") {
            var planetPoint = this.getPlanetPoint();
        }else {
            var planetPoint = this.getPlanetPointFromPastPoint(pastPoint);
        }
        var closerThenNeeded = false;
        for(var i = 0; i < this.planets.length; i++) {
            if(planetPoint.getDistance(this.planets[i].img.position) < settings.MIN_PLANET_DISTANCE) {
                closerThenNeeded = true;
                break;
            }
        }
        if(closerThenNeeded) continue;

        var planet = new Planet();
        this.planets.push(planet);
        planet.setToPoint(planetPoint);
//        console.log("x = " + Math.abs(constants.GAME_WIDTH / 2 - planetPoint.x) + "   y = " + Math.abs(constants.GAME_HEIGHT / 2 - planetPoint.y));
        planet.img.moveBelow(this.map.backRect);
    }
}

Space.prototype.getPlanetPointFromPastPoint = function(pastPoint) {
    var posX = 0;
    var posY = 0;
    if(pastPoint.x < 0 - settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_WIDTH / 2) {
        posX = constants.GAME_WIDTH / 2 + settings.ACTIVE_PLANETS_FIELD_SIZE - Math.round(Math.random() * settings.VARIATIVE_PLANET_SPAWN_COORDINATES);
        posY = Math.round(Math.random() * (settings.ACTIVE_PLANETS_FIELD_SIZE * 2)) - (settings.ACTIVE_PLANETS_FIELD_SIZE - constants.GAME_HEIGHT / 2);
    }else if(pastPoint.x > settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_WIDTH / 2) {
        posX = constants.GAME_WIDTH / 2 - settings.ACTIVE_PLANETS_FIELD_SIZE + Math.round(Math.random() * settings.VARIATIVE_PLANET_SPAWN_COORDINATES);
        posY = Math.round(Math.random() * (settings.ACTIVE_PLANETS_FIELD_SIZE * 2)) - (settings.ACTIVE_PLANETS_FIELD_SIZE - constants.GAME_HEIGHT / 2);
    }else if(pastPoint.y < 0 - settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_HEIGHT / 2) {
        posX = Math.round(Math.random() * (settings.ACTIVE_PLANETS_FIELD_SIZE * 2)) - (settings.ACTIVE_PLANETS_FIELD_SIZE - constants.GAME_WIDTH /2);
        posY = constants.GAME_HEIGHT / 2 + settings.ACTIVE_PLANETS_FIELD_SIZE - Math.round(Math.random() * settings.VARIATIVE_PLANET_SPAWN_COORDINATES);
    }else if(pastPoint.y > settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_HEIGHT / 2) {
        posX = Math.round(Math.random() * (settings.ACTIVE_PLANETS_FIELD_SIZE * 2)) - (settings.ACTIVE_PLANETS_FIELD_SIZE - constants.GAME_WIDTH /2);
        posY = constants.GAME_HEIGHT / 2 - settings.ACTIVE_PLANETS_FIELD_SIZE + Math.round(Math.random() * settings.VARIATIVE_PLANET_SPAWN_COORDINATES);
    }
    return new paper.Point(posX, posY);
}

Space.prototype.getPlanetPoint = function() {
    var xPos = Math.round(Math.random() * (settings.ACTIVE_PLANETS_FIELD_SIZE * 2)) - (settings.ACTIVE_PLANETS_FIELD_SIZE - constants.GAME_WIDTH /2);
    var yPos = Math.round(Math.random() * (settings.ACTIVE_PLANETS_FIELD_SIZE * 2)) - (settings.ACTIVE_PLANETS_FIELD_SIZE - constants.GAME_HEIGHT / 2);
    return new paper.Point(xPos, yPos);
}

Space.prototype.onFrame = function() {
    this.nova.onFrame();
    if(!objects.game.player.isFlying || typeof(objects.game.player.xDiference) == "undefined") return;
    var i = 0;
    while(i < this.planets.length) {
        this.planets[i].img.position.x += objects.game.player.xDiference;
        this.planets[i].img.position.y += objects.game.player.yDiference;
        this.planets[i].mapImg.position = new paper.Point(this.planets[i].img.position.x / 20 + constants.GAME_WIDTH - settings.ACTIVE_PLANETS_FIELD_SIZE / 10 / 2 - constants.GAME_WIDTH / 10 / 4,
                this.planets[i].img.position.y /20 + constants.GAME_HEIGHT - settings.ACTIVE_PLANETS_FIELD_SIZE / 10 * 2 + settings.ACTIVE_PLANETS_FIELD_SIZE / 10 / 2 - constants.GAME_HEIGHT / 10 / 4);

        if (this.planetOutsideField(this.planets[i])) {
            var planetPoint = this.planets[i].img.position;
            this.planets[i].clear();
            this.planets.splice(i, 1);
            this.setNeededPlanets(planetPoint);
            continue;
        }

        if(this.planets[i] == objects.game.player.activePlanet) continue;
        if(this.planets[i].img.position.getDistance(objects.game.player.img.position) < this.planets[i].size / 2 + settings.PLAYER_HEIGHT /2) {
            objects.game.player.landToPlanet(this.planets[i]);
        }
        i++;
    }
}

Space.prototype.planetOutsideField = function(planet) {
    if(planet.img.position.x < 0 - settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_WIDTH / 2
            || planet.img.position.x > settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_WIDTH / 2
            || planet.img.position.y < 0 - settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_HEIGHT / 2
            || planet.img.position.y > settings.ACTIVE_PLANETS_FIELD_SIZE + constants.GAME_HEIGHT / 2) {
        return true;
    }
    return false;
}

Space.prototype.clear = function() {
    while(this.planets.length > 0) {
        this.planets[0].clear();
        this.planets.splice(0, 1);
    }
    this.nova.clear();
}